package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.system.service.data.sys_area.QuerySysAreaOutput;
import pps.cloud.system.service.data.sys_dictionary.GetSysDictionaryViewOutput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.base.entity.*;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_excel.ExcelDieselGeneratorTemplate;
import pps.core.base.service.data.base_excel.ExcelEnergyStorageDeviceTemplate;
import pps.core.base.service.data.base_excel.ExcelPhotovoltaicPlantTemplate;
import pps.core.base.service.data.base_excel.ExcelWellheadTemplate;
import pps.core.base.service.data.excel.ImportExcelInput;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.excel.config.CascadeWriteHandler;
import pps.core.common.excel.listener.ExcelListener;
import pps.core.common.excel.util.EasyExcelUtil;
import pps.core.common.excel.util.ExcelStyleTool;
import pps.core.common.excel.util.ValidationUtil;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XServiceResult;
import xstartup.data.XUploadFile;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * excel导入导出
 *
 * @author ZWT
 * @date 2024/05/07 09:49
 */
@XText("Excel导入导出")
@XService
public class ExcelService {

    /**
     * 光伏电站配置模板导出
     *
     * @param context 上下文
     * @return {@link XFileResult}
     */
    @XText("光伏电站配置模板导出")
    @XApiGet(anonymous = true)
    public XFileResult templatePhotovoltaicPlant(XContext context) {
        //查地区列表
        Map<String, List<QuerySysAreaOutput>> collect = ServiceUtil.getProvinceAndCityList(context, "陕西", "甘肃")
                .stream()
                .collect(Collectors.groupingBy(QuerySysAreaOutput::getProvince));
        //整理数据，放入一个Map中，key存放父地点，value 存放该地点下的子区域
        Map<String, List<String>> siteMap = new HashMap<>(collect.size());
        List<String> provNameList = new ArrayList<>();
        for (String province : collect.keySet()) {
            provNameList.add(province);
            siteMap.put(province, collect.get(province).stream()
                    .map(QuerySysAreaOutput::getCity)
                    .collect(Collectors.toList())
            );
        }
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/光伏电站配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .registerWriteHandler(new CascadeWriteHandler(provNameList, siteMap))
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(Collections.emptyList(),
                        EasyExcelUtil.writeSelectedSheet(ExcelPhotovoltaicPlantTemplate.class, 0, "光伏电站配置")
                ).finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 井口配置模板导出
     *
     * @param context 上下文
     * @return {@link XFileResult}
     */
    @XText("井口配置模板导出")
    @XApiGet(anonymous = true)
    public XFileResult templateWellhead(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/井口配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(Collections.emptyList(),
                        EasyExcelUtil.writeSelectedSheet(ExcelWellheadTemplate.class, 0, "井口配置")
                ).finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 柴发设备配置模板导出
     *
     * @param context 上下文
     * @return {@link XFileResult}
     */
    @XText("柴发设备配置模板导出")
    @XApiGet(anonymous = true)
    public XFileResult templateDieselGenerator(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/柴发设备配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(Collections.emptyList(),
                        EasyExcelUtil.writeSelectedSheet(ExcelDieselGeneratorTemplate.class, 0, "柴发设备配置")
                ).finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 储能设备配置模板导出
     *
     * @param context 上下文
     * @return {@link XFileResult}
     */
    @XText("储能设备配置模板导出")
    @XApiGet(anonymous = true)
    public XFileResult templateEnergyStorageDevice(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/储能设备配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(Collections.emptyList(),
                        EasyExcelUtil.writeSelectedSheet(ExcelEnergyStorageDeviceTemplate.class, 0, "储能设备配置")
                ).finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /*------------------------------------ 导入 ------------------------------------*/

    /**
     * 光伏电站配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("光伏电站配置导入")
    @XApiUpload
    public XServiceResult importPhotovoltaicPlant(XContext context, ImportExcelInput input) {
        List<ExcelPhotovoltaicPlantTemplate> list = this.getExcelDataAndCheck(input.getFile(), ExcelPhotovoltaicPlantTemplate.class,
                "电站名称", ExcelPhotovoltaicPlantTemplate::getStationName);
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        Map<String, Integer> city = ServiceUtil.getProvinceAndCityList(context, "陕西", "甘肃").stream()
                .collect(Collectors.toMap(QuerySysAreaOutput::getCity, QuerySysAreaOutput::getId));
        //封装并校验电站名称(校验电站名称是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BasePhotovoltaicPlantMapper.class)
                .selectList(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                        .eq(BaseModel::getIsDeleted, 1)
                ).stream()
                .map(BasePhotovoltaicPlantEnt::getStationName)
                .collect(Collectors.toSet());
        StringBuilder stringBuilder = new StringBuilder();
        List<BasePhotovoltaicPlantView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        for (ExcelPhotovoltaicPlantTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getStationName())) {
                if (0 == stringBuilder.length()) {
                    stringBuilder.append("数据导入成功: ");
                }
                stringBuilder.append("第").append(rowNum).append("行,[电站名称重复]");
            } else {
                //数据封装
                BasePhotovoltaicPlantView build = BasePhotovoltaicPlantView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .areaCode(city.get(ent.getCity()))
                        .stationName(ent.getStationName())
                        .makerNumber(ent.getMakerNumber())
                        .photovoltaicModelName(ent.getPhotovoltaicModelName())
                        .mountingAngle(new BigDecimal(ent.getMountingAngle()))
                        .totalPower(new BigDecimal(ent.getTotalPower()))
                        .arrayOrientation(new BigDecimal(ent.getArrayOrientation()))
                        .longitude(new BigDecimal(ent.getLongitude()))
                        .latitude(new BigDecimal(ent.getLatitude()))
                        .elevation(new BigDecimal(ent.getElevation()))
                        .build();
                BaseUtils.createBaseModelDefault(build, context.getSession(PpsUserSession.class));
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BasePhotovoltaicPlantViewMapper viewMapper = context.getBean(BasePhotovoltaicPlantViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (stringBuilder.length() > 0) {
                return XServiceResult.error(-1, stringBuilder.toString());
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 井口配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("井口配置导入")
    @XApiUpload
    public XServiceResult importWellhead(XContext context, ImportExcelInput input) {
        List<ExcelWellheadTemplate> list = this.getExcelDataAndCheck(input.getFile(), ExcelWellheadTemplate.class,
                "井号", ExcelWellheadTemplate::getWellNumber);
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        Map<String, String> dictMap = ServiceUtil.getDictionaryList(context, BusinessConstant.RUN_TYPE).stream()
                .collect(Collectors.toMap(GetSysDictionaryViewOutput::getDicName, GetSysDictionaryViewOutput::getDicKey));
        //封装并校验井号(校验井号是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BaseWellheadMapper.class)
                .selectList(new LambdaQueryWrapper<BaseWellheadEnt>()
                        .eq(BaseModel::getIsDeleted, 1)
                ).stream()
                .map(BaseWellheadEnt::getWellNumber)
                .collect(Collectors.toSet());
        StringBuilder stringBuilder = new StringBuilder();
        List<BaseWellheadView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        for (ExcelWellheadTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getWellNumber())) {
                if (0 == stringBuilder.length()) {
                    stringBuilder.append("数据导入成功: ");
                }
                stringBuilder.append("第").append(rowNum).append("行,[井号重复]");
            } else {
                //数据封装
                BaseWellheadView build = BaseWellheadView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .wellNumber(ent.getWellNumber())
                        .runTypeName(ent.getRunTypeName())
                        .runTypeKey(dictMap.get(ent.getRunTypeName()))
                        .serviceRating(new BigDecimal(ent.getServiceRating()))
                        .ratedPower(new BigDecimal(ent.getRatedPower()))
                        .build();
                BaseUtils.createBaseModelDefault(build, context.getSession(PpsUserSession.class));
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BaseWellheadViewMapper viewMapper = context.getBean(BaseWellheadViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (stringBuilder.length() > 0) {
                return XServiceResult.error(-1, stringBuilder.toString());
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 柴发设备配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("柴发设备配置导入")
    @XApiUpload
    public XServiceResult importDieselGenerator(XContext context, ImportExcelInput input) {
        List<ExcelDieselGeneratorTemplate> list = this.getExcelDataAndCheck(input.getFile(), ExcelDieselGeneratorTemplate.class,
                "设备名称", ExcelDieselGeneratorTemplate::getDeviceName);
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        //封装并校验设备名称(校验设备名称是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BaseDieselGeneratorMapper.class)
                .selectList(new LambdaQueryWrapper<BaseDieselGeneratorEnt>()
                        .eq(BaseModel::getIsDeleted, 1)
                ).stream()
                .map(BaseDieselGeneratorEnt::getDeviceName)
                .collect(Collectors.toSet());
        StringBuilder stringBuilder = new StringBuilder();
        List<BaseDieselGeneratorView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        for (ExcelDieselGeneratorTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getDeviceName())) {
                if (0 == stringBuilder.length()) {
                    stringBuilder.append("数据导入成功: ");
                }
                stringBuilder.append("第").append(rowNum).append("行,[设备名称重复]");
            } else {
                //数据封装
                BaseDieselGeneratorView build = BaseDieselGeneratorView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .deviceName(ent.getDeviceName())
                        .makerNumber(ent.getMakerNumber())
                        .unitTypeName(ent.getUnitTypeName())
                        .normalPower(new BigDecimal(ent.getNormalPower()))
                        .standbyPower(new BigDecimal(ent.getStandbyPower()))
                        .build();
                BaseUtils.createBaseModelDefault(build, context.getSession(PpsUserSession.class));
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BaseDieselGeneratorViewMapper viewMapper = context.getBean(BaseDieselGeneratorViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (stringBuilder.length() > 0) {
                return XServiceResult.error(-1, stringBuilder.toString());
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 储能设备配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("储能设备配置导入")
    @XApiUpload(anonymous = true)
    public XServiceResult importEnergyStorageDevice(XContext context, ImportExcelInput input) {
        List<ExcelEnergyStorageDeviceTemplate> list = this.getExcelDataAndCheck(input.getFile(), ExcelEnergyStorageDeviceTemplate.class,
                "设备名称", ExcelEnergyStorageDeviceTemplate::getDeviceName);
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        //封装并校验设备名称(校验设备名称是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BaseEnergyStorageDeviceMapper.class)
                .selectList(new LambdaQueryWrapper<BaseEnergyStorageDeviceEnt>()
                        .eq(BaseModel::getIsDeleted, 1)
                ).stream()
                .map(BaseEnergyStorageDeviceEnt::getDeviceName)
                .collect(Collectors.toSet());
        StringBuilder stringBuilder = new StringBuilder();
        List<BaseEnergyStorageDeviceView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        for (ExcelEnergyStorageDeviceTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getDeviceName())) {
                if (0 == stringBuilder.length()) {
                    stringBuilder.append("数据导入成功: ");
                }
                stringBuilder.append("第").append(rowNum).append("行,[设备名称重复]");
            } else {
                //数据封装
                BaseEnergyStorageDeviceView build = BaseEnergyStorageDeviceView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .deviceName(ent.getDeviceName())
                        .makerNumber(ent.getMakerNumber())
                        .storageModelName(ent.getStorageModelName())
                        .apparatusCapacity(new BigDecimal(ent.getApparatusCapacity()))
                        .ratedDischargePower(new BigDecimal(ent.getRatedDischargePower()))
                        .ratedChargingPower(new BigDecimal(ent.getRatedChargingPower()))
                        .ratedDischargeDepth(new BigDecimal(ent.getRatedDischargeDepth()))
                        .ratedDischargeEfficiency(new BigDecimal(ent.getRatedDischargeEfficiency()))
                        .build();
                BaseUtils.createBaseModelDefault(build, context.getSession(PpsUserSession.class));
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BaseEnergyStorageDeviceViewMapper viewMapper = context.getBean(BaseEnergyStorageDeviceViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (stringBuilder.length() > 0) {
                return XServiceResult.error(-1, stringBuilder.toString());
            }
            return XServiceResult.OK;
        });
    }

    /*------------------------------------ private ------------------------------------*/

    /**
     * 获取excel数据并检查
     *
     * @param file       文件
     * @param head       头
     * @param headerName 标头名称
     * @param classifier 分类器
     * @return {@link List }<{@link T }>
     */
    private <T> List<T> getExcelDataAndCheck(XUploadFile file, Class<T> head, String headerName, Function<? super T, ? extends String> classifier) {
        this.checkFileTypeIsExcel(file.getFileName());
        ExcelListener<T> excelListener = new ExcelListener<>(-1);
        EasyExcel.read(file.getInputStream(), head, excelListener).sheet().doRead();
        List<T> dataList = excelListener.getDataList();
        this.checkExcelDataSize(dataList);
        ValidationUtil.doValidate(dataList);
        //检查某个名称字段是否重复
        List<String> duplicate = dataList.stream()
                .collect(Collectors.groupingBy(classifier, Collectors.counting()))
                .entrySet()
                .stream()
                .filter(e -> e.getValue() > 1)
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
        if (CollUtil.isNotEmpty(duplicate)) {
            String join = CharSequenceUtil.join(",", duplicate);
            throw new XServiceException("表格内[" + headerName + "]列, 名称为[" + join + "]重复");
        }
        return dataList;
    }

    /**
     * 检查文件类型为excel
     *
     * @param fileName 文件名
     */
    private void checkFileTypeIsExcel(String fileName) {
        // 截取文件后缀
        String fileSuffix = null;
        if (CharSequenceUtil.isNotBlank(fileName)) {
            fileSuffix = CharSequenceUtil.subAfter(fileName, ".", true);
        }
        String[] parts = {"xlsx", "xls"};
        // 判断文件是否是excel文件
        if (!CharSequenceUtil.equalsAny(fileSuffix, parts)) {
            throw new XServiceException(BusinessError.FileFormatError);
        }
    }

    /**
     * 检查excel数据大小
     *
     * @param dataList 数据列表
     */
    private void checkExcelDataSize(List dataList) {
        if (CollUtil.isEmpty(dataList)) {
            throw new XServiceException(BusinessError.UndiscoveredData);
        }
    }

    /**
     * 获取采油厂地图
     *
     * @param context 上下文
     * @return {@link Map }<{@link String }, {@link String }>
     */
    private Map<String, String> getPlantMap(XContext context) {
        return ServiceUtil.getPlantList(context).stream()
                .collect(Collectors.toMap(GetSysOrganizationViewOutput::getOuName, GetSysOrganizationViewOutput::getId));
    }
}